# line removed because contains an absolute path that changes each time test tests are run

# line removed because contains an absolute path that changes each time test tests are run

$(BUILD_GEN_DIR)/curv.mk $(BUILD_GEN_DIR)/.curv.env $(BUILD_GEN_DIR)/curvcfgpkg.sv $(BUILD_GEN_DIR)/curvcfg.svh: \
  $(BUILD_CONFIG_DIR)/merged.toml \
  $(CURV_ROOT_DIR)/tools/curvcfg/test/inputs/good/seperate_base_schema/default.toml
