// -----------------------------------------------------------------------------
// Autogenerated by curvcfg.py. Do not edit.
// -----------------------------------------------------------------------------

package curvcfgpkg;

  // verilator lint_off UNUSEDPARAM
  localparam logic [31:0] CFG_CACHE_HEX_FILES_BASE_ADDR = 32'h00000100;
  localparam string CFG_CACHE_HEX_FILES_SRC_NAME = "count1234";
  localparam string CFG_CACHE_HEX_FILES_SRC_TYPE = "asm";
  localparam string CFG_CACHE_HEX_FILES_SUBDIRS_ICACHE = "icache";
  localparam int CFG_CACHE_SETS = 4;
  localparam int CFG_CACHE_TAGS_IN_LUTRAM = 1;
  localparam logic [31:0] CFG_CPU_MTVEC_BASE = 32'h12345678;
  localparam logic [31:0] CFG_CPU_RESET_VECTOR = 32'h00000000;
  localparam int CFG_TB_CACHE_ICACHE_SIMULATE_MULTICYCLE_MEMORY_LATENCY = 0;
  // verilator lint_on UNUSEDPARAM

endpackage : curvcfgpkg
